@@ -5,6 +5,7 @@ TCPContainer and MQTTContainer
 from abc import ABC, abstractmethod
 import asyncio
 import logging
+import warnings
 from typing import Optional, Union, Tuple, Dict, Any, Set
 import paho.mqtt.client as paho
 from .container_protocols import ContainerProtocol
@@ -16,6 +17,7 @@ logger = logging.getLogger(__name__)
 
 AGENT_PATTERN_NAME_PRE = "agent"
 
+
 class Container(ABC):
     """Superclass for a mango container"""
 
@@ -263,7 +265,8 @@ class Container(ABC):
 
     def is_aid_available(self, aid):
         """
-        Check if the aid is available and registrable (it is not possible to register aids matching the regular pattern "agentX")
+        Check if the aid is available and allowed.
+        It is not possible to register aids matching the regular pattern "agentX".
         :param aid: the aid you want to check
         :return True if the aid is available, False if it is not
         """
@@ -311,34 +314,81 @@ class Container(ABC):
         receiver_addr: Union[str, Tuple[str, int]],
         *,
         receiver_id: Optional[str] = None,
-        create_acl: bool = False,
+        create_acl: bool = None,
         acl_metadata: Optional[Dict[str, Any]] = None,
         mqtt_kwargs: Dict[str, Any] = None,
+        **kwargs
     ) -> bool:
         """
-        container sends the message of one of its own agents to a specific topic
+        The Container sends a message to an agent according the container protocol.
+        
         :param content: The content of the message
         :param receiver_addr: In case of TCP this is a tuple of host, port
-        In case of MQTT this is the topic to publish to.
+            In case of MQTT this is the topic to publish to.
         :param receiver_id: The agent id of the receiver
         :param create_acl: True if an acl message shall be created around the
-        content.
+            content.
+            
+            .. deprecated:: 0.4.0
+                Use 'container.send_acl_message' instead. In the next version this parameter
+                will be dropped entirely.
         :param acl_metadata: metadata for the acl_header.
-        Ignored if create_acl == False
+            Ignored if create_acl == False
+            
+            .. deprecated:: 0.4.0
+                Use 'container.send_acl_message' instead. In the next version this parameter
+                will be dropped entirely.
         :param mqtt_kwargs: Dict with possible kwargs for publishing to a mqtt broker
             Possible fields:
             qos: The quality of service to use for publishing
             retain: Indicates, weather the retain flag should be set
             Ignored if connection_type != 'mqtt'
+            .. deprecated:: 0.4.0
+                Use 'kwargs' instead. In the next version this parameter
+                will be dropped entirely.
+        :param kwargs: Additional parameters to provide protocol specific settings 
         """
         raise NotImplementedError
 
+    async def send_acl_message(
+        self,
+        content,
+        receiver_addr: Union[str, Tuple[str, int]],
+        *,
+        receiver_id: Optional[str] = None,
+        acl_metadata: Optional[Dict[str, Any]] = None,
+        is_anonymous_acl=False,
+        **kwargs
+    ) -> bool:
+        """
+        The Container sends a message, wrapped in an ACL message, to an agent according the container protocol.
+
+        :param content: The content of the message
+        :param receiver_addr: In case of TCP this is a tuple of host, port
+        In case of MQTT this is the topic to publish to.
+        :param receiver_id: The agent id of the receiver
+        :param acl_metadata: metadata for the acl_header.
+        :param is_anonymous_acl: If set to True, the sender information won't be written in the ACL header 
+        :param kwargs: Additional parameters to provide protocol specific settings 
+        """
+        return await self.send_message(self._create_acl(
+            content,
+            receiver_addr=receiver_addr,
+            receiver_id=receiver_id,
+            acl_metadata=acl_metadata,
+            is_anonymous_acl=is_anonymous_acl),
+            receiver_addr=receiver_addr, 
+            receiver_id=receiver_id,
+            **kwargs
+        )
+
     def _create_acl(
         self,
         content,
         receiver_addr: Union[str, Tuple[str, int]],
         receiver_id: Optional[str] = None,
         acl_metadata: Optional[Dict[str, Any]] = None,
+        is_anonymous_acl=False
     ):
         """
         :param content:
@@ -347,39 +397,20 @@ class Container(ABC):
         :param acl_metadata:
         :return:
         """
-        acl_metadata = {} if acl_metadata is None else acl_metadata
+        acl_metadata = {} if acl_metadata is None else acl_metadata.copy()
         # analyse and complete acl_metadata
         if "receiver_addr" not in acl_metadata.keys():
             acl_metadata["receiver_addr"] = receiver_addr
         if "receiver_id" not in acl_metadata.keys() and receiver_id:
             acl_metadata["receiver_id"] = receiver_id
-        # add sender_addr if not defined
-        if "sender_addr" not in acl_metadata.keys() and self.addr is not None:
-            acl_metadata["sender_addr"] = self.addr
+        # add sender_addr if not defined and not anonymous
+        if not is_anonymous_acl:
+            if "sender_addr" not in acl_metadata.keys() and self.addr is not None:
+                acl_metadata["sender_addr"] = self.addr
 
         message = ACLMessage()
         message.content = content
 
-        # if self.codec == "json":
-        #     # create json message
-        #     message = json_ACLMessage()
-        #     message.content = content
-
-        # elif self.codec == "protobuf":
-        #     # create protobuf message
-        #     message = proto_ACLMessage()
-        #     receiver_meta = acl_metadata["receiver_addr"]
-        #     if isinstance(receiver_meta, (tuple, list)):
-        #         acl_metadata["receiver_addr"] = f"{receiver_meta[0]}:{receiver_meta[1]}"
-        #     sender_meta = acl_metadata.get("sender_addr", None)
-        #     if isinstance(sender_meta, (tuple, list)):
-        #         acl_metadata["sender_addr"] = f"{sender_meta[0]}:{sender_meta[1]}"
-
-        #     message.content_class = type(content).__name__
-        #     message.content = content.SerializeToString()
-        # else:
-        #     raise ValueError("Unknown Encoding")
-
         for key, value in acl_metadata.items():
             setattr(message, key, value)
         return message
@@ -402,20 +433,20 @@ class Container(ABC):
 
         while True:
             data = await self.inbox.get()
-            priority, msg_content, meta = data
+            priority, content, meta = data
             task = asyncio.create_task(
-                self._handle_msg(priority=priority, msg_content=msg_content, meta=meta)
+                self._handle_message(priority=priority, content=content, meta=meta)
             )
             task.add_done_callback(raise_exceptions)
             self.inbox.task_done()  # signals that the queue object is
             # processed
 
     @abstractmethod
-    async def _handle_msg(self, *, priority: int, msg_content, meta: Dict[str, Any]):
+    async def _handle_message(self, *, priority: int, content, meta: Dict[str, Any]):
         """
         This is called as a separate task for every message that is read
-        :param priority: priority of the msg
-        :param msg_content: Deserialized content of the message
+        :param priority: priority of the message
+        :param content: Deserialized content of the message
         :param meta: Dict with additional information (e.g. topic)
         """
         raise NotImplementedError
@@ -526,7 +557,7 @@ class MQTTContainer(Container):
 
         self.mqtt_client.on_subscribe = on_sub
 
-        def on_msg(client, userdata, message):
+        def on_message(client, userdata, message):
             # extract the meta information first
             meta = {
                 "network_protocol": "mqtt",
@@ -534,20 +565,20 @@ class MQTTContainer(Container):
                 "qos": message.qos,
                 "retain": message.retain,
             }
-            # decode message and extract msg_content and meta
-            msg_content, msg_meta = self.decode_mqtt_msg(
+            # decode message and extract content and meta
+            content, message_meta = self.decode_mqtt_message(
                 payload=message.payload, topic=message.topic
             )
             # update meta dict
-            meta.update(msg_meta)
+            meta.update(message_meta)
 
             # put information to inbox
-            if msg_content is not None:
+            if content is not None:
                 self.loop.call_soon_threadsafe(
-                    self.inbox.put_nowait, (0, msg_content, meta)
+                    self.inbox.put_nowait, (0, content, meta)
                 )
 
-        self.mqtt_client.on_message = on_msg
+        self.mqtt_client.on_message = on_message
 
         self.mqtt_client.enable_logger(logger)
 
@@ -560,9 +591,9 @@ class MQTTContainer(Container):
         self.mqtt_client.disconnect()
         self.mqtt_client.loop_stop()
 
-    def decode_mqtt_msg(self, *, topic, payload):
+    def decode_mqtt_message(self, *, topic, payload):
         """
-        deserializes a mqtt msg.
+        deserializes a mqtt message.
         Checks if for the topic a special class is defined, otherwise assumes
         an ACLMessage
         :param topic: the topic on which the message arrived
@@ -586,23 +617,23 @@ class MQTTContainer(Container):
 
         return decoded, meta
 
-    async def _handle_msg(self, *, priority: int, msg_content, meta: Dict[str, Any]):
+    async def _handle_message(self, *, priority: int, content, meta: Dict[str, Any]):
         """
         This is called as a separate task for every message that is read
-        :param priority: priority of the msg
-        :param msg_content: Deserialized content of the message
+        :param priority: priority of the message
+        :param content: Deserialized content of the message
         :param meta: Dict with additional information (e.g. topic)
 
         """
         topic = meta["topic"]
         logger.debug(
-            f"Received msg with content and meta;{str(msg_content)};{str(meta)}"
+            f"Received message with content and meta;{str(content)};{str(meta)}"
         )
 
-        if hasattr(msg_content, "split_content_and_meta"):
-            content, msg_meta = msg_content.split_content_and_meta()
-            meta.update(msg_meta)
-            msg_content = content
+        if hasattr(content, "split_content_and_meta"):
+            content, message_meta = content.split_content_and_meta()
+            meta.update(message_meta)
+            content = content
 
         if topic == self.inbox_topic:
             # General inbox topic, so no receiver is specified by the topic
@@ -610,7 +641,7 @@ class MQTTContainer(Container):
             receiver_id = meta.get("receiver_id", None)
             if receiver_id and receiver_id in self._agents.keys():
                 receiver = self._agents[receiver_id]
-                await receiver.inbox.put((priority, msg_content, meta))
+                await receiver.inbox.put((priority, content, meta))
             else:
                 logger.warning(f"Receiver ID is unknown;{receiver_id}")
         else:
@@ -627,7 +658,7 @@ class MQTTContainer(Container):
                 for receiver_id in receivers:
                     receiver = self._agents[receiver_id]
 
-                    await receiver.inbox.put((priority, msg_content, meta))
+                    await receiver.inbox.put((priority, content, meta))
 
     async def send_message(
         self,
@@ -635,25 +666,52 @@ class MQTTContainer(Container):
         receiver_addr: Union[str, Tuple[str, int]],
         *,
         receiver_id: Optional[str] = None,
-        create_acl: bool = False,
+        create_acl: bool = None,
         acl_metadata: Optional[Dict[str, Any]] = None,
         mqtt_kwargs: Dict[str, Any] = None,
+        **kwargs
     ):
         """
-        container sends the message of one of its own agents to a specific
-        topic
+        The container sends the message of one of its own agents to a specific topic.
+        
         :param content: The content of the message
-        :param receiver_addr: The topic that the message should be published to
+        :param receiver_addr: The topic to publish to.
         :param receiver_id: The agent id of the receiver
-        :param create_acl: True if the content is
-        only part of an acl message object that is yet to be created.
-        :param acl_metadata: metadata for the acl_header. Is only interpreted
-        if add_acl_header == True
-        :param mqtt_kwargs: Dict with possible kwargs for publishing to the
-        mqtt broker. Possible fields:
+        :param create_acl: True if an acl message shall be created around the
+            content.
+            
+            .. deprecated:: 0.4.0
+                Use 'container.send_acl_message' instead. In the next version this parameter
+                will be dropped entirely.
+        :param acl_metadata: metadata for the acl_header.
+            Ignored if create_acl == False
+            
+            .. deprecated:: 0.4.0
+                Use 'container.send_acl_message' instead. In the next version this parameter
+                will be dropped entirely.
+        :param mqtt_kwargs: Dict with possible kwargs for publishing to a mqtt broker
+            Possible fields:
             qos: The quality of service to use for publishing
             retain: Indicates, weather the retain flag should be set
+            Ignored if connection_type != 'mqtt'
+            .. deprecated:: 0.4.0
+                Use 'kwargs' instead. In the next version this parameter
+                will be dropped entirely.
+        :param kwargs: Additional parameters to provide protocol specific settings 
+            Possible fields:
+            qos: The quality of service to use for publishing
+            retain: Indicates, weather the retain flag should be set
+            Ignored if connection_type != 'mqtt'
+
         """
+
+        if create_acl is not None or acl_metadata is not None:
+            warnings.warn("The parameters create_acl and acl_metadata are deprecated and will "
+                          "be removed in the next release. Use send_acl_message instead.", DeprecationWarning)
+        if mqtt_kwargs is not None:
+            warnings.warn("The parameter mqtt_kwargs is deprecated and will "
+                          "be removed in the next release. Use kwargs instead.", DeprecationWarning)
+
         if create_acl:
             message = self._create_acl(
                 content=content,
@@ -667,22 +725,23 @@ class MQTTContainer(Container):
 
         # internal message first (if retain Flag is set, it has to be sent to
         # the broker
-        mqtt_kwargs = {} if mqtt_kwargs is None else mqtt_kwargs
+        actual_mqtt_kwargs = mqtt_kwargs if kwargs is None else kwargs
+        actual_mqtt_kwargs = {} if actual_mqtt_kwargs is None else actual_mqtt_kwargs
         if (
             self.addr
             and receiver_addr == self.addr
-            and not mqtt_kwargs.get("retain", False)
+            and not actual_mqtt_kwargs.get("retain", False)
         ):
             meta = {
                 "topic": self.addr,
-                "qos": mqtt_kwargs.get("qos", 0),
+                "qos": actual_mqtt_kwargs.get("qos", 0),
                 "retain": False,
                 "network_protocol": "mqtt",
             }
 
             if hasattr(message, "split_content_and_meta"):
-                content, msg_meta = message.split_content_and_meta()
-                meta.update(msg_meta)
+                content, message_meta = message.split_content_and_meta()
+                meta.update(message_meta)
             else:
                 content = message
 
@@ -700,15 +759,9 @@ class MQTTContainer(Container):
         :param message: The ACL message
         :return:
         """
-        encoded_msg = self.codec.encode(message)
-        # if self.codec == "json":
-        #     encoded_msg = message.encode()
-        # elif self.codec == "protobuf":
-        #     encoded_msg = message.SerializeToString()
-        # else:
-        #     raise ValueError("Unknown codec")
+        encoded_message = self.codec.encode(message)
         logger.debug(f"Sending message;{message};{topic}")
-        self.mqtt_client.publish(topic, encoded_msg)
+        self.mqtt_client.publish(topic, encoded_message)
 
     async def subscribe_for_agent(
         self, *, aid: str, topic: str, qos: int = 0, expected_class=None
@@ -808,21 +861,21 @@ class TCPContainer(Container):
         self.server = None  # will be set within the factory method
         self.running = True
 
-    async def _handle_msg(self, *, priority: int, msg_content, meta: Dict[str, Any]):
+    async def _handle_message(self, *, priority: int, content, meta: Dict[str, Any]):
         """
 
         :param priority:
-        :param msg_content:
+        :param content:
         :param meta:
         :return:
         """
         logger.debug(
-            f"Received msg with content and meta;{str(msg_content)};{str(meta)}"
+            f"Received message with content and meta;{str(content)};{str(meta)}"
         )
         receiver_id = meta.get("receiver_id", None)
         if receiver_id and receiver_id in self._agents.keys():
             receiver = self._agents[receiver_id]
-            await receiver.inbox.put((priority, msg_content, meta))
+            await receiver.inbox.put((priority, content, meta))
         else:
             logger.warning(f"Received a message for an unknown receiver;{receiver_id}")
 
@@ -832,22 +885,42 @@ class TCPContainer(Container):
         receiver_addr: Union[str, Tuple[str, int]],
         *,
         receiver_id: Optional[str] = None,
-        create_acl: bool = False,
+        create_acl: bool = None,
         acl_metadata: Optional[Dict[str, Any]] = None,
         mqtt_kwargs: Dict[str, Any] = None,
+        **kwargs
     ) -> bool:
         """
-        container sends the message of one of its own agents to a specific topic
+        The Container sends a message to an agent using TCP.
+        
         :param content: The content of the message
-        :param receiver_addr: In case of TCP this is a tuple of host, port
+        :param receiver_addr: Tuple of host, port
         :param receiver_id: The agent id of the receiver
         :param create_acl: True if an acl message shall be created around the
-        content.
+            content.
+            
+            .. deprecated:: 0.4.0
+                Use 'container.send_acl_message' instead. In the next version this parameter
+                will be dropped entirely.
         :param acl_metadata: metadata for the acl_header.
-        Ignored if create_acl == False
-        :param mqtt_kwargs: Ignored in this class
-        :return boolean indicating whether  sending was successful or not
+            Ignored if create_acl == False
+            
+            .. deprecated:: 0.4.0
+                Use 'container.send_acl_message' instead. In the next version this parameter
+                will be dropped entirely.
+        :param mqtt_kwargs: 
+            .. deprecated:: 0.4.0
+                Use 'kwargs' instead. In the next version this parameter
+                will be dropped entirely.
+        :param kwargs: Additional parameters to provide protocol specific settings 
         """
+        if create_acl is not None or acl_metadata is not None:
+            warnings.warn("The parameters create_acl and acl_metadata are deprecated and will "
+                          "be removed in the next release. Use send_acl_message instead.", DeprecationWarning)
+        if mqtt_kwargs is not None:
+            warnings.warn("The parameter mqtt_kwargs is deprecated and will "
+                          "be removed in the next release. Use kwargs instead.", DeprecationWarning)
+
         if isinstance(receiver_addr, str) and ":" in receiver_addr:
             receiver_addr = receiver_addr.split(":")
         elif isinstance(receiver_addr, (tuple, list)) and len(receiver_addr) == 2:
@@ -856,7 +929,7 @@ class TCPContainer(Container):
             logger.warning(f"Address for sending message is not valid;{receiver_addr}")
             return False
 
-        if create_acl:
+        if create_acl is not None and create_acl:
             message = self._create_acl(
                 content=content,
                 receiver_addr=receiver_addr,
